# Hebern5.py : une machine a chiffrer avec 5 rotors
# et il y a deux stators: clavier et lampe
# on peut inverser le rotor, utiliser le mode chiffre/dechiff
# et specifier la clef de messages (la position init du rotor

import string
import sys
from RotorBiFace import *
ALPHA = string.ascii_uppercase
# ===================================
def is_maj(car):
    if ALPHA.find(car) == -1:
        return False
    else:
        return True
# ==================================
def let2car( lettre ):
    return string.ascii_uppercase[ lettre ]
def car2let( car ):
    return string.index(string.ascii_uppercase, car.upper())
# ==================================
def litRot( un_rot ):
    f = open( "ROTORS/" + un_rot + ".rot" )
    alpha = f.readline()
    f.readline()
    nom = f.readline()
    f.close()
    return [ nom.rstrip(), alpha.rstrip() ]
# ==================================
def litfic( fic ):
    f = open(fic,"r")
    leTexte = f.readlines()
    texteConcat = ""
    f.close()
    for i in range(len(leTexte)):
        ch = leTexte[i]
        ch = ch.upper()
        for j in range(len(ch)):
            if not is_maj(ch[j]):
                continue
            texteConcat = texteConcat + ch[j]
    return texteConcat
# ================================================
class Hebern5:
    keyboard = "XAKHSZJLYWGPMIOURDBFTNVCQE"
    lampboard= "TYOEUMXDFJQVKWBNSHCILRZAGP"

    def __init__(self,intKey=["I","II","III","IV","V"],
                 verse=[0,0,0,0,0],mode=0,debug=0):
        self.Index = 0
        self.Mode = mode  #0: chiffre, 1: dechiffre
        self.Verse = verse
        self.StatorKeyb = ROTOR( Hebern5.keyboard )
        self.Rotors = []
        for i in range(5):
            [ nom, alpha ] = litRot( intKey[i] )
            self.Rotors.append( RotorBiFace(nom , alpha, verse[i], debug))
        self.StatorLamp = ROTOR( Hebern5.lampboard )
        self.Debug = debug
        self.DontMove = False
        self.RatchetGauche = 0  # A = 0, N = 13
        self.RatchetDroit  = 0
        self.DrapeauGauche = False
        self.DrapeauDroit  = False

    def info(self):
        ch = ""
        ch += "Cle: " + self.obtient_cle()  + "\n"
        ch += "Mode (0:chiffre,1:dechiffre):" + str(self.Mode) + "\n"
        ch += "Key Stator : " + self.StatorKeyb.Pi + "\n"
        for i in range(5):
            ch += self.Rotors[i].info()
        ch += "Lamp Stator : " + self.StatorLamp.Pi + "\n"
        return ch
        
    def mise_a_la_cle(self, cle ):
        indx_r = ALPHA.index( cle[0].upper() )
        self.RatchetGauche = indx_r
        for i in range(5): 
            self.Rotors[i].mise_a_la_cle( cle[ i+1 ].upper() )
        indx_r = ALPHA.index( cle[6].upper() )
        self.RatchetDroit = indx_r
        if self.RatchetDroit  == 13: 
            self.DrapeauDroit  = True
        else:
            self.DrapeauDroit  = False
        self.DrapeauGauche = False
        if self.DrapeauDroit and self.RatchetGauche == 13: 
            self.DrapeauGauche = True
        """
        if self.RatchetGauche == 13:
            self.DrapeauGauche = True
        """

    def obtient_cle(self ):
        cle = ""
        cle += string.ascii_uppercase[ self.RatchetGauche ]
        for i in range(5):
            cle += self.Rotors[i].obtient_cle()
        cle += string.ascii_uppercase[ self.RatchetDroit  ]
        return cle

    def toogleMode(self):
        if self.Mode == 0: self.Mode = 1
        else: self.Mode = 0
        
    def toogleDontMove(self):
        if self.DontMove == 0: self.DontMove = 1
        else: self.DontMove = 0

    def avance(self):
        # -- Ratchet Droit et Rotor V avancent
        self.RatchetDroit += 1
        if self.RatchetDroit == 26: self.RatchetDroit = 0
        self.Rotors[4].step()

        # -- Rotor III avance si le Ratchet Gauche a atteint N
        #    au stade precedent
        if self.DrapeauGauche :
            self.Rotors[2].step()
            self.DrapeauGauche = False
        #    self.RatchetGauche += 1

        # -- Rotor I et Ratchet Gauche avancent 
        #    Si au stade precedent Ratchet Droit a atteint N
        if self.DrapeauDroit :
            self.Rotors[0].step()
            self.DrapeauDroit = False
            self.RatchetGauche += 1
            if self.RatchetGauche == 26: self.RatchetGauche = 0
            if self.RatchetGauche == 13: self.DrapeauGauche = True

        if self.RatchetDroit  == 13: self.DrapeauDroit  = True
        
    def chiffre_un_car(self, car):
        self.avance()
        lettre_clair = ALPHA.index(car.upper())
        x = self.StatorKeyb.chiffre(lettre_clair)
        if self.Debug: 
            print(let2car(x),end="")
        for i in range(5):
            x = self.Rotors[i].chiffre( x )
            if self.Debug: print(let2car(x),end="")
        x = self.StatorLamp.chiffre( x )
        if self.Debug: print("->", let2car(x),end="")
        new_car = string.ascii_uppercase[ x ]
        return new_car
        
    def dechiffre_un_car(self, car):
        self.avance()
        lettre_clair = ALPHA.index(car.upper())
        x = self.StatorLamp.dechiffre(lettre_clair)
        if self.Debug: print(let2car(x),end="")
        for i in range(5):
            x = self.Rotors[4-i].dechiffre( x )
            if self.Debug: print(let2car(x),end="")
        x = self.StatorKeyb.dechiffre( x )
        if self.Debug: print("->", let2car(x),end="")
        new_car = string.ascii_uppercase[ x ]
        return new_car
        
    def chiffre(self, texte ):
        ch = ""
        for i in range( len(texte) ):
            self.Index += 1
            if self.Debug:
                print("%03d: " % (self.Index),end="")
                print(texte[i], "->", end="")
            if not self.Mode :
                c = self.chiffre_un_car( texte[i] )
            else:
                c = self.dechiffre_un_car( texte[i] )
            ch += c
            if self.Debug:
                print("   ", self.obtient_cle())
        return ch
        
    def rewind(self):
        self.Index = 0
        for i in range(5):
            self.Rotors[i].rewind()
        self.RatchetGauche = 0
        self.RatchetDroit  = 0
        
#========================================================================
if __name__ == "__main__" :
    h = Hebern5(["III","I","V","II","IV"],debug=1)
        
    print(h.info())
    h.mise_a_la_cle("MEANING")
    print("Cle: ", h.obtient_cle())
    clair = "ILFAITBEAUCEMATIN"
    crypto = h.chiffre( clair )
    print("Crypto: ", crypto)
    print("Cle: ", h.obtient_cle())

    print("====> dechiffre")
    #h.rewind()
    h.mise_a_la_cle("MEANING")
    h.toogleMode()
    print("Cle: ", h.obtient_cle())
    clair2 = h.chiffre( crypto )
    print("Clair:", clair2)

